import { TriggerIntegration, IO, ConnectionAuth, Json, IntegrationTaskKey, IOTask, RunTaskOptions, RunTaskErrorCallback, OverridableRunTaskOptions, ExternalSource, EventFilter, ExternalSourceTrigger, EventSpecification } from '@trigger.dev/sdk';
import { SupabaseManagementAPI, GetOrganizationsResponseData, GetProjectsResponseData, CreateProjectRequestBody, CreateProjectResponseData, ListFunctionsResponseData, RunQueryResponseData, GetTypescriptTypesResponseData, GetPostgRESTConfigResponseData, GetProjectPGConfigResponseData } from 'supabase-management-js';
import { Prettify } from '@trigger.dev/integration-kit';
import { SupabaseClientOptions, SupabaseClient } from '@supabase/supabase-js';

type GenericTable = {
    Row: Record<string, unknown>;
    Insert: Record<string, unknown>;
    Update: Record<string, unknown>;
};
type GenericUpdatableView = {
    Row: Record<string, unknown>;
    Insert: Record<string, unknown>;
    Update: Record<string, unknown>;
};
type GenericNonUpdatableView = {
    Row: Record<string, unknown>;
};
type GenericView = GenericUpdatableView | GenericNonUpdatableView;
type GenericFunction = {
    Args: Record<string, unknown>;
    Returns: unknown;
};
type GenericSchema = {
    Tables: Record<string, GenericTable>;
    Views: Record<string, GenericView>;
    Functions: Record<string, GenericFunction>;
};

type SupabaseManagementIntegrationOptions = {
    id: string;
} | {
    id: string;
    apiKey: string;
};
declare class SupabaseDatabase<Database = any> {
    private integration;
    private projectRef;
    constructor(integration: SupabaseManagement, projectRef: string);
    /**
     * The function `on` creates a trigger for when a record is inserted, updated, or deleted on a
     * specific table in a database schema.
     * @param params - The `params` parameter is an object that contains the following properties:
     * @param params.table - The `table` property is a string that specifies the name of the table
     * that the trigger will be created for.
     * @param params.events - The `events` property is an array of events that specifies the events
     * that the trigger will be called for. The events that can be specified are `INSERT`, `UPDATE`, or `DELETE`.
     * By default, the trigger will be called for all events.
     * @param params.schema - The `schema` property is a string that specifies the name of the schema
     * that the trigger will be created for. If the schema is not specified, the default schema will
     * be used. (public)
     * @param params.filter - The `filter` property is an object that specifies the filter that will
     * be used to determine if the trigger should be called. If the filter is not specified, the
     * trigger will be called for all records.
     *
     * @example
     *
     * ```ts
     * const supabase = new SupabaseManagement({ id: "supabase" });
     * const database = supabase.database<Database>("https://<project-id>.supabase.co");
     *
     * client.defineJob({
     *  trigger: database.on({
     *    table: "todos",
     *    events: ["INSERTED", "UPDATED"],
     *    schema: "public",
     *    filter: {
     *     record: { is_completed: [false] },
     *    },
     *  }),
     * })
     * ```
     */
    on<SchemaName extends string & keyof Database = "public" extends keyof Database ? "public" : string & keyof Database, Schema extends GenericSchema = Database[SchemaName] extends GenericSchema ? Database[SchemaName] : any, TTableName extends string & keyof Schema["Tables"] = string & keyof Schema["Tables"], TTable extends Schema["Tables"][TTableName] = Schema["Tables"][TTableName], TEvents extends WebhookEvents[] = ["INSERT", "UPDATE", "DELETE"]>(params: {
        table: TTableName;
        events?: TEvents;
        schema?: SchemaName;
        filter?: EventFilter;
    }): ExternalSourceTrigger<EventSpecification<UnionPayloads<TEvents, TTableName, SchemaName, TTable["Row"]> extends infer T ? { [K in keyof T]: UnionPayloads<TEvents, TTableName, SchemaName, TTable["Row"]>[K]; } : never, UnionPayloads<TEvents, TTableName, SchemaName, TTable["Row"]> extends infer T ? { [K in keyof T]: UnionPayloads<TEvents, TTableName, SchemaName, TTable["Row"]>[K]; } : never>, ExternalSource<SupabaseManagement, {
        projectRef: string;
        table: string;
        schema: string;
    }, "HTTP", {}>>;
    /**
     * The function `onInserted` creates a trigger for when a new record is inserted into a specific
     * table in a database schema.
     * @param params - The `params` parameter is an object that contains the following properties:
     * @param params.table - The `table` property is a string that specifies the name of the table
     * that the trigger will be created for.
     * @param params.schema - The `schema` property is a string that specifies the name of the schema
     * that the trigger will be created for. If the schema is not specified, the default schema will
     * be used. (public)
     * @param params.filter - The `filter` property is an object that specifies the filter that will
     * be used to determine if the trigger should be called. If the filter is not specified, the
     * trigger will be called for all records.
     *
     * @example
     *
     * ```ts
     * const supabase = new SupabaseManagement({ id: "supabase" });
     * const database = supabase.database<Database>("https://<project-id>.supabase.co");
     *
     * client.defineJob({
     *  trigger: database.onInserted({
     *    table: "todos",
     *    schema: "public",
     *    filter: {
     *     record: { is_completed: [false] },
     *    },
     *  }),
     * })
     * ```
     */
    onInserted<SchemaName extends string & keyof Database = "public" extends keyof Database ? "public" : string & keyof Database, Schema extends GenericSchema = Database[SchemaName] extends GenericSchema ? Database[SchemaName] : any, TTableName extends string & keyof Schema["Tables"] = string & keyof Schema["Tables"], TTable extends Schema["Tables"][TTableName] = Schema["Tables"][TTableName]>(params: {
        table: TTableName;
        schema?: SchemaName;
        filter?: EventFilter;
    }): ExternalSourceTrigger<EventSpecification<{
        table: TTableName;
        record: Prettify<TTable["Row"]>;
        type: "INSERT";
        schema: SchemaName;
        old_record: null;
    }, {
        table: TTableName;
        record: Prettify<TTable["Row"]>;
        type: "INSERT";
        schema: SchemaName;
        old_record: null;
    }>, ExternalSource<SupabaseManagement, {
        projectRef: string;
        table: string;
        schema: string;
    }, "HTTP", {}>>;
    /**
     * The function `onUpdated` creates a trigger for when a new record is updated on a specific
     * table in a database schema.
     * @param params - The `params` parameter is an object that contains the following properties:
     * @param params.table - The `table` property is a string that specifies the name of the table
     * that the trigger will be created for.
     * @param params.schema - The `schema` property is a string that specifies the name of the schema
     * that the trigger will be created for. If the schema is not specified, the default schema will
     * be used. (public)
     * @param params.filter - The `filter` property is an object that specifies the filter that will
     * be used to determine if the trigger should be called. If the filter is not specified, the
     * trigger will be called for all records.
     *
     * @example
     *
     * ```ts
     * const supabase = new SupabaseManagement({ id: "supabase" });
     * const database = supabase.database<Database>("https://<project-id>.supabase.co");
     *
     * client.defineJob({
     *  trigger: database.onUpdated({
     *    table: "todos",
     *    schema: "public",
     *    filter: {
     *     record: { completed: [true] },
     *     old_record: { completed: [false] },
     *    },
     *  }),
     * })
     * ```
     */
    onUpdated<SchemaName extends string & keyof Database = "public" extends keyof Database ? "public" : string & keyof Database, Schema extends GenericSchema = Database[SchemaName] extends GenericSchema ? Database[SchemaName] : any, TTableName extends string & keyof Schema["Tables"] = string & keyof Schema["Tables"], TTable extends Schema["Tables"][TTableName] = Schema["Tables"][TTableName]>(params: {
        table: TTableName;
        schema?: SchemaName;
        filter?: EventFilter;
    }): ExternalSourceTrigger<EventSpecification<{
        table: TTableName;
        record: Prettify<TTable["Row"]>;
        type: "UPDATE";
        schema: SchemaName;
        old_record: Prettify<TTable["Row"]>;
    }, {
        table: TTableName;
        record: Prettify<TTable["Row"]>;
        type: "UPDATE";
        schema: SchemaName;
        old_record: Prettify<TTable["Row"]>;
    }>, ExternalSource<SupabaseManagement, {
        projectRef: string;
        table: string;
        schema: string;
    }, "HTTP", {}>>;
    /**
     * The function `onDeleted` creates a trigger for when a new record is deleted from a specific
     * table in a database schema.
     * @param params - The `params` parameter is an object that contains the following properties:
     * @param params.table - The `table` property is a string that specifies the name of the table
     * that the trigger will be created for.
     * @param params.schema - The `schema` property is a string that specifies the name of the schema
     * that the trigger will be created for. If the schema is not specified, the default schema will
     * be used. (public)
     * @param params.filter - The `filter` property is an object that specifies the filter that will
     * be used to determine if the trigger should be called. If the filter is not specified, the
     * trigger will be called for all records.
     *
     * @example
     *
     * ```ts
     * const supabase = new SupabaseManagement({ id: "supabase" });
     * const database = supabase.database<Database>("https://<project-id>.supabase.co");
     *
     * client.defineJob({
     *  trigger: database.onDeleted({
     *    table: "todos",
     *    schema: "public",
     *    filter: {
     *     old_record: { is_completed: [true] },
     *    },
     *  }),
     * })
     * ```
     */
    onDeleted<SchemaName extends string & keyof Database = "public" extends keyof Database ? "public" : string & keyof Database, Schema extends GenericSchema = Database[SchemaName] extends GenericSchema ? Database[SchemaName] : any, TTableName extends string & keyof Schema["Tables"] = string & keyof Schema["Tables"], TTable extends Schema["Tables"][TTableName] = Schema["Tables"][TTableName]>(params: {
        table: TTableName;
        schema?: SchemaName;
        filter?: EventFilter;
    }): ExternalSourceTrigger<EventSpecification<{
        table: TTableName;
        record: null;
        type: "DELETE";
        schema: SchemaName;
        old_record: Prettify<TTable["Row"]>;
    }, {
        table: TTableName;
        record: null;
        type: "DELETE";
        schema: SchemaName;
        old_record: Prettify<TTable["Row"]>;
    }>, ExternalSource<SupabaseManagement, {
        projectRef: string;
        table: string;
        schema: string;
    }, "HTTP", {}>>;
}
declare class SupabaseManagement implements TriggerIntegration {
    private options;
    constructor(options: SupabaseManagementIntegrationOptions);
    get id(): string;
    get metadata(): {
        id: string;
        name: string;
    };
    get source(): WebhookEventSource;
    get authSource(): "LOCAL" | "HOSTED";
    cloneForRun(io: IO, connectionKey: string, auth?: ConnectionAuth): SupabaseManagement;
    runTask<T, TResult extends Json<T> | void>(key: IntegrationTaskKey, callback: (client: SupabaseManagementAPI, task: IOTask, io: IO) => Promise<TResult>, options?: RunTaskOptions, errorCallback?: RunTaskErrorCallback): Promise<TResult>;
    /**
     * Creates a new database instance that can be used to listen to changes in the database.
     *
     * @param projectIdOrUrl The project ID or URL of the Supabase project (e.g. `https://<project-id>.supabase.co`)
     * @param options Options for the database instance
     */
    db<Database = any>(projectIdOrUrl: string): SupabaseDatabase<Database>;
    getOrganizations(key: IntegrationTaskKey): Promise<GetOrganizationsResponseData>;
    getProjects(key: IntegrationTaskKey): Promise<GetProjectsResponseData>;
    createProject(key: IntegrationTaskKey, params: CreateProjectRequestBody): Promise<CreateProjectResponseData>;
    listFunctions(key: IntegrationTaskKey, params: {
        ref: string;
    }): Promise<ListFunctionsResponseData>;
    runQuery(key: IntegrationTaskKey, params: {
        ref: string;
        query: string;
    }): Promise<RunQueryResponseData>;
    getTypescriptTypes(key: IntegrationTaskKey, params: {
        ref: string;
    }): Promise<GetTypescriptTypesResponseData>;
    getPostgRESTConfig(key: IntegrationTaskKey, params: {
        ref: string;
    }): Promise<GetPostgRESTConfigResponseData>;
    /** Gets project's Postgres config */
    getPGConfig(key: IntegrationTaskKey, params: {
        ref: string;
    }): Promise<GetProjectPGConfigResponseData>;
    /** Enable Database Webhooks in project */
    enableDatabaseWebhooks(key: IntegrationTaskKey, params: {
        ref: string;
    }, options?: OverridableRunTaskOptions): Promise<void>;
}
type WebhookEventSource = ReturnType<typeof createWebhookEventSource>;
type WebhookEvents = "INSERT" | "UPDATE" | "DELETE";
type WebhookEventPayloads<TTableName extends string, TSchemaName extends string = "public", TRecord = any> = {
    INSERT: {
        table: TTableName;
        record: Prettify<TRecord>;
        type: "INSERT";
        schema: TSchemaName;
        old_record: null;
    };
    UPDATE: {
        table: TTableName;
        record: Prettify<TRecord>;
        type: "UPDATE";
        schema: TSchemaName;
        old_record: Prettify<TRecord>;
    };
    DELETE: {
        table: TTableName;
        record: null;
        type: "DELETE";
        schema: TSchemaName;
        old_record: Prettify<TRecord>;
    };
};
type UnionPayloads<T extends WebhookEvents[], TTableName extends string, TSchemaName extends string = "public", TRecord = any> = WebhookEventPayloads<TTableName, TSchemaName, TRecord>[T[number]];
declare function createWebhookEventSource(integration: SupabaseManagement): ExternalSource<SupabaseManagement, {
    projectRef: string;
    table: string;
    schema: string;
}, "HTTP", {}>;

type SupabaseIntegrationOptions<TSchema extends string> = {
    /** The unique ID for this integration */
    id: string;
    /** The Supabase project url (e.g. "https://<project-id>.supabase.co") */
    supabaseUrl: string;
    /** The Supabase service account API Key (found in your Supabase Project Settings -> API -> service_role) */
    supabaseKey: string;
    /** Options that are passed through to the call the createClient */
    options?: SupabaseClientOptions<TSchema>;
} | {
    id: string;
    projectId: string;
    supabaseKey: string;
    options?: SupabaseClientOptions<TSchema>;
};
/**
 * A Trigger Integration for Supabase
 *
 * @example
 * ```ts
 * import { Supabase } from "@trigger.dev/supabase";
 * import { Database } from "@/supabase.types";
 *
 * const supabase = new Supabase<Database>({
 *  id: "my-supabase",
 *  projectId: process.env.SUPABASE_ID!,
 *  supabaseKey: process.env.SUPABASE_API_KEY!,
 * });
 * ```
 */
declare class Supabase<Database = any, SchemaName extends string & keyof Database = "public" extends keyof Database ? "public" : string & keyof Database, Schema extends GenericSchema = Database[SchemaName] extends GenericSchema ? Database[SchemaName] : any> implements TriggerIntegration {
    private options;
    /**
     * The native Supabase client. This is exposed for use outside of Trigger.dev jobs
     *
     * @example
     * ```ts
     * import { Supabase } from "@trigger.dev/supabase";
     * import { Database } from "@/supabase.types";
     *
     * const supabase = new Supabase<Database>({
     *   id: "my-supabase",
     *   projectId: process.env.SUPABASE_ID!,
     *   supabaseKey: process.env.SUPABASE_API_KEY!,
     * });
     *
     * const { data, error } = await supabase.native.from("users").select("*");
     * ```
     */
    readonly native: SupabaseClient<Database, SchemaName, Schema>;
    constructor(options: SupabaseIntegrationOptions<SchemaName>);
    get authSource(): "LOCAL";
    get id(): string;
    get metadata(): {
        id: string;
        name: string;
    };
    get client(): SupabaseClient<Database, SchemaName, Schema>;
    cloneForRun(io: IO, connectionKey: string, auth?: ConnectionAuth): Supabase<Database, SchemaName, Schema>;
    runTask<T, TResult extends Json<T> | void>(key: IntegrationTaskKey, callback: (client: SupabaseClient<Database, SchemaName, Schema>, task: IOTask, io: IO) => Promise<TResult>, options?: RunTaskOptions, errorCallback?: RunTaskErrorCallback): Promise<TResult>;
}

export { Supabase, type SupabaseIntegrationOptions, SupabaseManagement, type SupabaseManagementIntegrationOptions, createWebhookEventSource };
